package com.android.sample.tvusdkdemo;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.SpannableString;
import android.text.style.ForegroundColorSpan;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.tvunetworks.android.sdk.TVUAnywhereSDK;
import com.tvunetworks.android.sdk.view.CustomConfigView;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends Activity {
    private static final String TAG = MainActivity.class.getSimpleName();
    private static final long TIME_PERIOD = 500L;
    private CustomConfigView configurationView;
    private Button startPreview, pairToken;
    private TextView tvPeerId;
    private Switch onlyModeSwitch;
    private long mLastClickTime;
    private int clickTimes;
    private SharedPreferences preferences;

    private static final String[] REQUIRED_PERMISSION_LIST = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.RECORD_AUDIO,
            Manifest.permission.READ_PHONE_STATE,
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION,
    };
    private List<String> missingPermission = new ArrayList<>();
    private static final int REQUEST_PERMISSION_CODE = 112233;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        getActionBar().setDisplayShowHomeEnabled(false);
        getActionBar().setDisplayShowTitleEnabled(false);

        preferences = getSharedPreferences(RegisterActivity.KEY_REGISTER_PREFERENCES, Context.MODE_PRIVATE);

        startPreview = (Button) findViewById(R.id.start_preview);
        pairToken = (Button) findViewById(R.id.btn_pair_token);
        tvPeerId = (TextView) findViewById(R.id.tv_peerId);
        onlyModeSwitch = findViewById(R.id.only_mode_switch);
        onlyModeSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                TVUAnywhereSDK.getInstance().tvuSetPartylineOnlyMode(isChecked);
            }
        });

        if (!hasPermissionsGranted()) {
            checkAndRequestPermissions();
            return;
        }

        initTVUSDKAfterPermissionsGranted();
        Log.d(TAG, "onCreate: ");
    }

    private void initTVUSDKAfterPermissionsGranted() {
        TVUAnywhereSDK.getInstance().tvuInitWhenAppStarted(MainActivity.this);
        // set sdk environment with api and environment is US by default if not set
//        TVUAnywhereSDK.getInstance().tvuSetTVUAnywhereSDKEnvironment(TVUEnvironment.ENVIRONMENT_US);
        // set sdk log level and the level is ERROR by default
//        TVUAnywhereSDK.getInstance().tvuSetAppLogLevel(TVULogLevel.LOG_LEVEL_DEBUG);

        TVUAnywhereSDK.getInstance().registerTVUAnywhereSDKWithAppKey(
                preferences.getString(RegisterActivity.KEY_APP_KEY, ""),
                preferences.getString(RegisterActivity.KEY_APP_SECRET, ""),
                preferences.getString(RegisterActivity.KEY_USER_NAME, ""),
                preferences.getString(RegisterActivity.KEY_USER_PASSWORD, ""));

        tvPeerId.setText(String.format("SDK Version: %s\nPeer ID: %s",
                TVUAnywhereSDK.getInstance().tvuGetSDKVersion(),
                TVUAnywhereSDK.getInstance().tvuGetMyPeerID()));
    }

    /**
     * Check if permissions granted
     *
     * @return true or false
     */
    private boolean hasPermissionsGranted() {
        for (String permission : REQUIRED_PERMISSION_LIST) {
            if (ActivityCompat.checkSelfPermission(this, permission)
                    != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    /**
     * Checks if there is any missing permissions, and
     * requests runtime permission if needed.
     */
    private void checkAndRequestPermissions() {
        // Check for permissions
        for (String eachPermission : REQUIRED_PERMISSION_LIST) {
            if (ContextCompat.checkSelfPermission(this, eachPermission) != PackageManager.PERMISSION_GRANTED) {
                missingPermission.add(eachPermission);
            }
        }
        // Request for missing permissions
        if (!missingPermission.isEmpty() && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            ActivityCompat.requestPermissions(this,
                    missingPermission.toArray(new String[missingPermission.size()]),
                    REQUEST_PERMISSION_CODE);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        // Check for granted permission and remove from missing list
        if (requestCode == REQUEST_PERMISSION_CODE) {
            for (int i = grantResults.length - 1; i >= 0; i--) {
                if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                    missingPermission.remove(permissions[i]);
                }
            }
        }
        // If there is enough permission, we will init sdk
        if (!missingPermission.isEmpty()) {
            Toast.makeText(this, "Missing permissions!!!", Toast.LENGTH_SHORT).show();
        }
        initTVUSDKAfterPermissionsGranted();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main_activity, menu);
        final MenuItem menuItem = menu.findItem(R.id.register_app_key);
        SpannableString spannableString = new SpannableString(menuItem.getTitle());
        spannableString.setSpan(new ForegroundColorSpan(Color.BLUE), 0, spannableString.length(), 0);
        menuItem.setTitle(spannableString);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.register_app_key) {
            // start activity to register app key and secret
            startActivity(new Intent(MainActivity.this, RegisterActivity.class));
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.d(TAG, "onStart: ");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.d(TAG, "onResume: ");
    }

    @Override
    protected void onPause() {
        super.onPause();
        Log.d(TAG, "onPause: ");
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.d(TAG, "onStop: ");
        TVUAnywhereSDK.getInstance().tvuReleaseWhenAppDestroyed(MainActivity.this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.d(TAG, "onDestroy: ");
        System.exit(0);
    }

    public void startDefaultUi(View v) {
//        TVUAnywhereSDK.getInstance().enableBackgroundRunning(true);
        TVUAnywhereSDK.getInstance().startDefaultActivity(MainActivity.this);
    }

    public void startPreview(View v) {
        startPreview.setEnabled(false);
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                Intent intent = new Intent(MainActivity.this, PreviewActivity.class);
                startActivity(intent);
                startPreview.setEnabled(true);
            }
        }, 500L);
    }

    public void pairToken(View v) {
        startActivity(new Intent(this, PairTokenActivity.class));
    }

    public void changeConfig(View v) {
        long currentClickTime = System.currentTimeMillis();//current time
        long elapsedTime = currentClickTime - mLastClickTime;//interval time
        if (elapsedTime <= TIME_PERIOD) {
            clickTimes += 1;
        } else {
            clickTimes = 1;
        }
        mLastClickTime = currentClickTime;//last time
        Log.d(TAG, "clickTimes:" + clickTimes + ", elapsedTime:" + elapsedTime);
        if (clickTimes == 10 && hasPermissionsGranted()) {
            showConfigDialog();
        }
    }

    private void showConfigDialog() {
        final AlertDialog.Builder layout_builder = new AlertDialog.Builder(MainActivity.this);
        configurationView = new CustomConfigView(MainActivity.this, null);
        layout_builder.setTitle(R.string.configuration)
                .setView(configurationView)
                .setPositiveButton(R.string.apply_configuration, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        Toast.makeText(MainActivity.this, "Configuration changed, restart app needed", Toast.LENGTH_SHORT).show();
                        configurationView.saveConfiguration();
                        MainActivity.this.finish();
                    }
                })
                .show();
    }
}
