package com.android.sample.tvusdkdemo;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.tvunetworks.android.sdk.TVUAnywhereSDK;
import com.tvunetworks.android.sdk.TVUServiceCallback;

public class PairTokenActivity extends Activity {
    private EditText etReceiver;
    private Button btnAddPair, btnRemovePair;
    private SharedPreferences preferences;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_token_pair);
        getActionBar().setDisplayShowHomeEnabled(false);

        preferences = getSharedPreferences(RegisterActivity.KEY_REGISTER_PREFERENCES, Context.MODE_PRIVATE);

        etReceiver = (EditText) findViewById(R.id.et_receiver_id);
        btnAddPair = (Button) findViewById(R.id.btn_add_pair);
        btnRemovePair = (Button) findViewById(R.id.btn_remove_pair);

        etReceiver.setText(preferences.getString(RegisterActivity.KEY_RECEIVER_ID, RegisterActivity.KEY_RD_RECEIVER_ID));

        btnAddPair.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                addOrRemovePair(v);
            }
        });

        btnRemovePair.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                addOrRemovePair(v);
            }
        });
    }

    private void showMessage(final String msg) {
        PairTokenActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                btnAddPair.setEnabled(true);
                btnRemovePair.setEnabled(true);
                Toast.makeText(PairTokenActivity.this, msg, Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void addOrRemovePair(final View view) {
        final CharSequence receiver = etReceiver.getText();
        if (TextUtils.isEmpty(receiver)) {
            Toast.makeText(PairTokenActivity.this, "Please input receiver id", Toast.LENGTH_LONG).show();
            return;
        }

        final String receiverId = receiver.toString().trim();
        if (TextUtils.isEmpty(receiverId)) {
            Toast.makeText(PairTokenActivity.this, "Please input the legal receiver id", Toast.LENGTH_LONG).show();
            return;
        }

        preferences.edit().putString(RegisterActivity.KEY_RECEIVER_ID, receiverId).apply();

        final String[] receiverIds = receiverId.split(",");

        switch (view.getId()) {
            case R.id.btn_add_pair:
                btnAddPair.setEnabled(false);
                btnRemovePair.setEnabled(false);
                TVUAnywhereSDK.getInstance().tvuAddTokenPairingWithPeerId(receiverIds, new TVUServiceCallback() {
                    @Override
                    public void onResult(final String s) {
                        showMessage("add token pairing: " + s);
                    }
                });
                break;
            case R.id.btn_remove_pair:
                btnAddPair.setEnabled(false);
                btnRemovePair.setEnabled(false);
                TVUAnywhereSDK.getInstance().tvuRemoveTokenPairingWithPeerId(receiverIds, new TVUServiceCallback() {
                    @Override
                    public void onResult(String s) {
                        showMessage("remove token pairing: " + s);
                    }
                });
                break;
            default:
                break;
        }
    }
}
