package com.android.sample.tvusdkdemo;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.tvunetworks.android.anywhere.stream.PushStreamService;
import com.tvunetworks.android.sdk.TVUAnywhereSDK;
import com.tvunetworks.android.sdk.TVUSDKCallback;
import com.tvunetworks.android.sdk.TVUServiceCallback;
import com.tvunetworks.android.sdk.model.TVULiveState;

import java.util.Arrays;
import java.util.Timer;
import java.util.TimerTask;

public class PreviewActivity extends Activity implements View.OnClickListener {
    private static final String TAG = PreviewActivity.class.getSimpleName();
    private static final int CAMERA_FACING_BACK = 0;
    private static final int CAMERA_FACING_FRONT = 1;
    private GLSurfaceView glSurfaceView;
    private SeekBar beautySeekBar;
    private TextView tvLiveBitrate, tvTVUVoice;
    private Button btnLive, btnOperateFlash, btnTakePicture, btnSwitchCamera, btnBeauty, btnHangupVoIP;
    private ImageView ivBack;
    private int live_status = 0;
    private boolean isFlashLightOn = false;
    private String last_r_list = "";
    private Timer mCheckRListTimer;
    private View pushStreamView;
    private EditText etRtmpUrl;
    private SharedPreferences preferences;
    private int cameraId = CAMERA_FACING_BACK;
    private AlertDialog voiceDialog;
    private View voiceContentView;
    private boolean tvuVoiceStatus = false;
    private TimerTask mCheckRListTask = new TimerTask() {
        @Override
        public void run() {
            final String rList = TVUAnywhereSDK.getInstance().tvuGetReceiverNameList();
            live_status = TVUAnywhereSDK.getInstance().tvuGetLivingReceiverStatus();
            Log.d(TAG, "R list is :" + rList);
            PreviewActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (TextUtils.isEmpty(rList)) {
                        btnLive.setText(getText(R.string.str_start_live));
                        btnLive.setEnabled(false);
                        tvLiveBitrate.setVisibility(View.INVISIBLE);
                        last_r_list = "";
                    } else if (!last_r_list.equals(rList)) {
                        last_r_list = rList;
                        btnLive.setEnabled(true);
                    }

                    if (live_status == 1) {
                        tvLiveBitrate.setText(String.format(getString(R.string.str_live_info),
                                TVUAnywhereSDK.getInstance().tvuGetLivingReceiverName(),
                                TVUAnywhereSDK.getInstance().tvuGetLiveBitrate()));
                    }

                    // check whether TVU is in calling status to show or hide tip view
                    final boolean curVoiceStatus = TVUAnywhereSDK.getInstance().tvuIsTVUVoicing();
                    if (tvuVoiceStatus != curVoiceStatus) {
                        if (curVoiceStatus) {
                            showVoIPTipView();
                        } else {
                            hideVoIPTipView();
                        }
                        tvuVoiceStatus = curVoiceStatus;
                    }
                    Log.d(TAG, "Voice Call Status: " + TVUAnywhereSDK.getInstance().tvuQueryVoiceCallStatus());
                }
            });
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_preview);

        preferences = getSharedPreferences(RegisterActivity.KEY_REGISTER_PREFERENCES, Context.MODE_PRIVATE);

        glSurfaceView = (GLSurfaceView) findViewById(R.id.surfaceview);
        beautySeekBar = findViewById(R.id.beauty_bar);
        btnLive = (Button) findViewById(R.id.live_button);
        btnOperateFlash = (Button) findViewById(R.id.btn_operate_flash);
        btnTakePicture = (Button) findViewById(R.id.btn_take_picture);
        btnSwitchCamera = (Button) findViewById(R.id.btn_switch_camera);
        btnBeauty = findViewById(R.id.btn_beauty);
        btnHangupVoIP = (Button) findViewById(R.id.btn_hangupVoIP);
        tvTVUVoice = (TextView) findViewById(R.id.tv_tvu_voice);
        tvLiveBitrate = (TextView) findViewById(R.id.tv_live_info);
        ivBack = findViewById(R.id.return_main_activity);
        ivBack.setOnClickListener(this);
        btnLive.setEnabled(false);
        btnLive.setOnClickListener(this);
        btnOperateFlash.setOnClickListener(this);
        btnTakePicture.setOnClickListener(this);
        btnSwitchCamera.setOnClickListener(this);
        btnBeauty.setOnClickListener(this);
        btnHangupVoIP.setOnClickListener(this);

        pushStreamView = findViewById(R.id.layout_push_stream);
        etRtmpUrl = findViewById(R.id.et_rtmp_url);
        etRtmpUrl.setText(preferences.getString(RegisterActivity.KEY_RTMP_URL, ""));
        findViewById(R.id.btn_start_push).setOnClickListener(this);
        findViewById(R.id.btn_stop_push).setOnClickListener(this);

        beautySeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                TVUAnywhereSDK.getInstance().adjustBeautyLevel(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        if (null == mCheckRListTimer) {
            mCheckRListTimer = new Timer("Timer-checkRList");
            mCheckRListTimer.schedule(mCheckRListTask, 1000, 1000);
        }

        TVUAnywhereSDK.getInstance().tvuCreated(this, glSurfaceView, true, new TVUCallback());
        Log.d(TAG, String.format("Device peerId is [%s]",
                TVUAnywhereSDK.getInstance().tvuGetMyPeerID()));
        Log.d(TAG, "dumpServiceConfigInfo: " + TVUAnywhereSDK.getInstance().dumpServiceConfigInfo());
        Log.d(TAG, "onCreate: ");

        // set callback to listen for the coming tvu voice and show dialog in the callback
        TVUAnywhereSDK.getInstance().setTVUVoiceComingCallback(new TVUServiceCallback() {
            @Override
            public void onResult(String s) {
                Log.d(TAG, "setTVUVoiceComingCallback: coming call name = " + s);
                showVoIPDialog(s);
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        TVUAnywhereSDK.getInstance().tvuSetParameters(1280, 720, 30);
        TVUAnywhereSDK.getInstance().tvuResume(cameraId);

        hideOrShowFlashButton();
        Log.d(TAG, "onResume: resolution list :"  + Arrays.toString(TVUAnywhereSDK.getInstance().tvuGetResolutionList())
                + ", frame rate list :" + Arrays.toString(TVUAnywhereSDK.getInstance().tvuGetFrameRateList()));
    }

    @Override
    protected void onPause() {
        super.onPause();
        TVUAnywhereSDK.getInstance().tvuPause();
        Log.d(TAG, "onPause: ");
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.d(TAG, "onStop: ");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        cancelCheckRListTimer();
        TVUAnywhereSDK.getInstance().tvuDestroy();
        Log.d(TAG, "onDestroy: ");
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.live_button:
                int result = 0;
                btnLive.setEnabled(false);
                String curRName = TVUAnywhereSDK.getInstance().tvuGetLivingReceiverName();
                if (TextUtils.isEmpty(curRName)) {
                    curRName = last_r_list.split(",")[0];
                }
                live_status = TVUAnywhereSDK.getInstance().tvuGetLivingReceiverStatus();
                if (live_status == 0) {
                    result = TVUAnywhereSDK.getInstance().tvuStartLive(curRName);
                } else {
                    result = TVUAnywhereSDK.getInstance().tvuStopLive(curRName);
                }
                if (result == -1) {
                    Log.d(TAG, "onClick: receiver name is empty");
                } else if (result == 1) {
                    Log.d(TAG, "onClick: NTP offset is wrong");
                } else {
                    Log.d(TAG, "onClick: click to start or stop live success");
                }
                break;
            case R.id.btn_operate_flash:
                if (TVUAnywhereSDK.getInstance().tvuOperateFlash(isFlashLightOn)) {
                    isFlashLightOn = !isFlashLightOn;
                } else {
                    Toast.makeText(this, "not support flash for current camera", Toast.LENGTH_LONG).show();
                }
                if (isFlashLightOn) {
                    btnOperateFlash.setText(R.string.str_flash_off);
                } else {
                    btnOperateFlash.setText(R.string.str_flash_on);
                }
                Log.d(TAG, "onClick: click to operate flash");
                break;
            case R.id.btn_take_picture:
                TVUAnywhereSDK.getInstance().tvuTakePicture();
                Toast.makeText(this, R.string.str_take_picture_tip, Toast.LENGTH_LONG).show();
                Log.d(TAG, "onClick: click to take picture");
                break;
            case R.id.btn_switch_camera:
                btnSwitchCamera.setEnabled(false);
                btnOperateFlash.setVisibility(View.INVISIBLE);
                cameraId = cameraId == CAMERA_FACING_BACK ? CAMERA_FACING_FRONT : CAMERA_FACING_BACK;
                if (!TVUAnywhereSDK.getInstance().tvuSwitchCamera(cameraId)) {
                    // if switch camera failed, reset camera id
                    cameraId = CAMERA_FACING_BACK;
                    Toast.makeText(this, "current device do not support to switch camera", Toast.LENGTH_LONG).show();
                    break;
                }
                btnSwitchCamera.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        btnSwitchCamera.setEnabled(true);
                        hideOrShowFlashButton();
                    }
                }, 1500);
                Log.d(TAG, "onClick: click to switch camera, current camera id=" + cameraId);
                break;
            case R.id.btn_start_push:
                startPush(String.valueOf(etRtmpUrl.getText()).trim());
                break;
            case R.id.btn_stop_push:
                stopPush(String.valueOf(etRtmpUrl.getText()).trim());
            case R.id.btn_hangupVoIP:
                TVUAnywhereSDK.getInstance().tvuHangupTVUVoice();
                break;
            case R.id.btn_beauty:
                if (TVUAnywhereSDK.getInstance().isBeauty()) {
                    TVUAnywhereSDK.getInstance().closeBeauty();
                    btnBeauty.setText(R.string.str_start_beauty);
                    beautySeekBar.setVisibility(View.GONE);
                } else {
                    TVUAnywhereSDK.getInstance().openBeauty();
                    btnBeauty.setText(R.string.str_stop_beauty);
                    beautySeekBar.setVisibility(View.VISIBLE);
                }
                break;
            case R.id.return_main_activity:
                finish();
                break;
            default:
                break;
        }
    }

    private void cancelCheckRListTimer() {
        if (mCheckRListTimer != null) {
            mCheckRListTimer.cancel();
            mCheckRListTimer = null;
        }
    }

    private void hideOrShowFlashButton() {
        if (cameraId == CAMERA_FACING_BACK) {
            isFlashLightOn = false;
            btnOperateFlash.setText(R.string.str_flash_on);
            btnOperateFlash.setVisibility(View.VISIBLE);
        } else {
            btnOperateFlash.setVisibility(View.INVISIBLE);
        }
    }

    private void startPush(String rtmpUrl) {
        if (TextUtils.isEmpty(rtmpUrl)) {
            Toast.makeText(this, "Please input valid rtmp url", Toast.LENGTH_SHORT).show();
            return;
        }

        preferences.edit().putString(RegisterActivity.KEY_RTMP_URL, rtmpUrl).apply();

        TVUAnywhereSDK.getInstance().tvuStartPushStreamToRtmpURL(rtmpUrl, new PushStreamService.Callback() {
            @Override
            public void onResult(String s) {
                Log.d(TAG, "onResult: " + s);
                showToastOnMainUI(s);
            }
        });
    }

    private void stopPush(String rtmpUrl) {
        if (TextUtils.isEmpty(rtmpUrl)) {
            Toast.makeText(this, "Please input valid rtmp url", Toast.LENGTH_SHORT).show();
            return;
        }

        preferences.edit().putString(RegisterActivity.KEY_RTMP_URL, rtmpUrl).apply();

        TVUAnywhereSDK.getInstance().tvuStopPushStreamToRtmpURL(rtmpUrl, new PushStreamService.Callback() {
            @Override
            public void onResult(String s) {
                Log.d(TAG, "onResult: " + s);
                showToastOnMainUI(s);
            }
        });
    }

    private void initVoIPDialog() {
        if (voiceDialog == null) {
            voiceContentView = LayoutInflater.from(this).inflate(R.layout.layout_call_voip, null);
            voiceDialog = new AlertDialog.Builder(this).setView(voiceContentView).create();
            voiceDialog.setCanceledOnTouchOutside(false);
            voiceDialog.setCancelable(false);
        }
    }

    private void showVoIPDialog(final String phoneName) {
        PreviewActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                initVoIPDialog();
                ((TextView) voiceContentView.findViewById(R.id.tv_phoneName)).setText(phoneName);
                voiceContentView.findViewById(R.id.btn_acceptVoIP).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        hideVoIPDialog();
                        TVUAnywhereSDK.getInstance().tvuAcceptTVUVoice();
                    }
                });
                voiceContentView.findViewById(R.id.btn_rejectVoIP).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        hideVoIPDialog();
                        TVUAnywhereSDK.getInstance().tvuRejectTVUVoice();
                    }
                });
                voiceDialog.show();
            }
        });
    }

    private void hideVoIPDialog() {
        PreviewActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (voiceDialog != null && voiceDialog.isShowing()) {
                    voiceDialog.dismiss();
                    voiceDialog = null;
                }
            }
        });
    }

    private void showPushStreamView() {
        if (pushStreamView.getVisibility() != View.VISIBLE) {
            pushStreamView.setVisibility(View.VISIBLE);
        }
    }

    private void hidePushStreamView() {
        if (pushStreamView.getVisibility() == View.VISIBLE) {
            pushStreamView.setVisibility(View.INVISIBLE);
        }
    }

    private void showToastOnMainUI(final String message) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(PreviewActivity.this, message, Toast.LENGTH_LONG).show();
            }
        });
    }

    private class TVUCallback implements TVUSDKCallback {
        @Override
        public void onStartLive(TVULiveState tvuLiveState) {
            btnLive.setEnabled(true);
            switch (tvuLiveState) {
                case SUCCESS:
                    btnLive.setText(getText(R.string.str_stop_live));
                    tvLiveBitrate.setVisibility(View.VISIBLE);
                    break;
                case BUSY:
                    showToastOnMainUI("Receiver is living with other T");
                    break;
                default:
                    break;
            }
        }

        @Override
        public void onStopLive(TVULiveState tvuLiveState) {
            btnLive.setEnabled(true);
            switch (tvuLiveState) {
                case SUCCESS:
                    btnLive.setText(getText(R.string.str_start_live));
                    tvLiveBitrate.setVisibility(View.INVISIBLE);
                    break;
                default:
                    break;
            }
        }

        @Override
        public void onMsgReceived(String s, int i, int i1) {

        }
    }

    private void showVoIPTipView() {
        PreviewActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                tvTVUVoice.setVisibility(View.VISIBLE);
                btnHangupVoIP.setVisibility(View.VISIBLE);
            }
        });
    }

    private void hideVoIPTipView() {
        PreviewActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                tvTVUVoice.setVisibility(View.GONE);
                btnHangupVoIP.setVisibility(View.GONE);
            }
        });
    }
}
